/*
 *  COPYRIGHT (c) 2012 by Lattice Semiconductor Corporation
 *
 * All rights reserved. All use of this software and documentation is
 * subject to the License Agreement located in the file LICENSE.
 */

/**
 * @file main.c
 * 
 * Test code for the XO2 I2C Embedded Configuration Access routines.
 * This provides the various demos/tests that demonstrate using I2C access to 
 * the XO2 on the Pico board.
 * <p>
 * This code is specific to porting the XO2 ECA to the Lattice Mico32 platform
 * running on the Lattice ECP3 Versa eval board.  It is not generic code.
 * It is specific exmaple implementation of the XO2 ECA.
 * <p>
 * The test examples show the following:
 * <ul>
 * <li> I2C bus access to Pico Board (read a temperature sensor chip)
 * <li> Read XO2 Configuration information via I2C
 * <li> Program a converted JEDEC file into the XO2 flash via I2C
 * <li> Program new EBR values into just the UFM, while design continues operating
 * <li> Individual UFM page accesses while design is running.
 * </ul>
 * <code>
 * <pre>
 * ----------------------           ---------------------     
 * |  ECP3 Versa Board  |           |   XO2 Pico Board  |
 * |    -------------   |           |                   |
 * |    |  ECP3      |  |           |  ---------------- |
 * |    |            |  |           |  |   XO2        | |
 * |    |     Mico32 |  |           |  |       -> Regs| |
 * |    |       I2C  |<-|-----------|->| I2C----> Cfg | |
 * |    -------------   |           |  |       -> UFM | | 
 * ----------------------           ---------------------
 * </pre>
 * </code>
 */
 
#include <stdio.h>
#include <stddef.h>


#include "DDStructs.h"
#include "system_conf.h"
#include "OpenCoresI2CMaster.h"
#include "MicoUtils.h"
#include "MicoUart.h"
#include "systypes.h"

#include "XO2_cmds.h"
#include "ECA_drvr.h"
#include "oc_i2c_port.h"
#include "XO2_api.h"

//-----------------------------------------------------------------------------------------
// Uncomment to enable menu option to select Offline mode.  By default all operations
// are done in Transparent mode because Offline mode is not supported with a standard
// XO2 Pico board.  A hardware mod must be made to the Pico board to enable I2C power
// at all times, which includes Offline mode and when programming a blank part.
// See RD1129.pdf for details on the mod.
//-----------------------------------------------------------------------------------------
//#define ENABLE_ALL_MODES 

////////////////////////////////////////////////////////////////////////////////////////
//       XO2 JEDEC FILE DATA
////////////////////////////////////////////////////////////////////////////////////////
#include "ufm_0123.h"
#include "ufm_DEAD.h"


#define UFM_START_PAGE 256   // Start here for user data (beginning is EBR init vals)



////////////////////////////////////////////////////////////////////////////////////////
//                     GLOBAL VARIABLES 
////////////////////////////////////////////////////////////////////////////////////////

volatile unsigned char *pLEDs = (volatile unsigned char *)GPIO_LED_BASE_ADDRESS;
volatile unsigned char *pSEGs = (volatile unsigned char *)GPIO_14SEG_BASE_ADDRESS;
volatile unsigned char *pSw = (volatile unsigned char *)GPIO_DIPSW_BASE_ADDRESS;
MicoUartCtx_t *UARTctx;

int ProgramMode;
int VerifyMode;

unsigned char UFMPagePattern;
 
unsigned int UFMNextPage;

const char *JEDECDesignList[3] = 
{
	"???",
	"ufm_0123(number pattern)",
	"ufm_DEAD(word pattern)"
};


extern 
////////////////////////////////////////////////////////////////////////////////////////
//                      FUNCTION PROTOTYPES
////////////////////////////////////////////////////////////////////////////////////////

void XO2_i2c_tests(void);
void temp_sensor_i2c_test(OpenCoresI2CMasterCtx_t *i2c_ctx, int loop);
void XO2_i2c_readback(void);
int XO2_i2c_prog(XO2_JEDEC_t *pProgJED, int doCfg);
void LEDtest(int loop);
char menu(int);
void UFMPageDataExchange(XO2Handle_t *pXO2dev);
void	readAndDisplayPage(XO2Handle_t *pXO2dev, unsigned int ufmPg);

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// BUG IN MICO CODE
// have this function prototypes wrong in OpenCoresI2CMaster.h
extern void OpenCoresI2CMasterDisable( OpenCoresI2CMasterCtx_t *ctx );
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!



/////////////////////////////////////////////////////
//    MAIN   ENTRY
/////////////////////////////////////////////////////

/**
 * This code provides an example of how to use the XO2 ECA API's 
 * to access the UFM, Config, and registers in the XO2 via the 
 * I2C bus.
 */
int main() 
{
	int designNum;
	int status;
	char selection;
	int display;
	int done;
	XO2RegInfo_t regInfo;
	unsigned int statusReg;


	XO2_JEDEC_t *pCfgBits;  // pointer to the JEDEC confgiruation data to load into XO2

	// Mico32 OpenCores I2C handle for I2C driver routines
	OpenCoresI2CMasterCtx_t *i2c_ctx;    
	
	// Declare the handle to the XO2 device that we will be accessing
	XO2Handle_t xo2_dev;

	// The Embedded Config Access access layer drivers that need to be populated
	// with the platform specific dirvers for the specific link, i.e. I2C
	ECADrvrCalls_t i2c_drvr;

	// Declare holder for the OpenCores I2C driver parameters needed by the OpenCores drivers
	// that need to be passed along with each call from an XO2 command routine.
	OC_I2C_t oc_i2c;


	// Clear LEDs
	*pLEDs = 0x00;

	UARTctx = (MicoUartCtx_t *)MicoGetDevice(UART_NAME);
	if (UARTctx == NULL)
	{
		printf("ERROR!  UART driver not found.\r\n");
		return(-1);
	}

	//--------------------------------------------------------------
	// First: 
	// Initialize the OpenCores I2C driver
	//--------------------------------------------------------------
	i2c_ctx = (OpenCoresI2CMasterCtx_t *)MicoGetDevice(I2CM_OC_NAME);
	if (i2c_ctx == NULL)
	{
		printf("ERROR!  OpenCores I2C controller not found.\r\n");
		return(-1);
	}

	// This function enables the OpenCores I2C master component instance
	// specified by the device context parameter (ctx).
	OpenCoresI2CMasterEnable(i2c_ctx);
	

	//--------------------------------------------------------------
	// Second: 
	// Populate the OpenCores driver specific structures to initialize
	// XO2 ECA with.
	//--------------------------------------------------------------
	// Populate the list of functions to call for the specific driver for this
	// specific hardware: OpenCores I2C controller running on Mico32 platform
	i2c_drvr.pfCmdStart = &oc_i2c_start;
	i2c_drvr.pfCmdStop = &oc_i2c_stop;
	i2c_drvr.pfCmdRead = &oc_i2c_read;
	i2c_drvr.pfCmdWrite = &oc_i2c_write;
	i2c_drvr.pfmSecDelay = &MicoSleepMilliSecs;
	i2c_drvr.pfuSecDelay = &MicoSleepMicroSecs;

	// Fill in the OpenCores context and the XO2 slave address for use by drivers
	oc_i2c.slaveAddr =  0x40;   // blank XO2 default I2C address (7 bit world), becomes 0x80
	oc_i2c.i2c_ctx = i2c_ctx;   // save I2C controller context for use in driver


	//--------------------------------------------------------------
	// Third: 
	// Initialize the XO2 ECA handle with the I2C driver fucntions
	// and other settings for the specific XO2 to access.
	//--------------------------------------------------------------
	xo2_dev.devType = MachXO2_1200;   // must match the real device
	xo2_dev.cfgEn = 0;  // initial value - config not enabled at start (could also be done with an API init call)
	xo2_dev.pDrvrCalls = &i2c_drvr;   // driver routines to start/stop read/write over I2C
	xo2_dev.pDrvrParams = (void *)&oc_i2c;   // extra info to pass onto the driver routines
	


	//--------------------------------------------------------------
	// Fourth: 
	// Invoke XO2 ECA APIs via menu selections.
	//--------------------------------------------------------------

	// Initial conditions
	ProgramMode = XO2ECA_PROGRAM_TRANSPARENT;
	VerifyMode = 0;  // no verify
	designNum = 1;   // numbers JEDEC design
	pCfgBits  = &XO2_JEDEC_ufm_0123;
	display = 1;
	done = 0;
	UFMPagePattern = 0;
	UFMNextPage = UFM_START_PAGE;   // Start here for user data (beginning is EBR init vals)

	printf("\033[2J\r\n");   // clear terminal
	printf("\r\n===================================================\r\n");
	printf(    "ECP3 I2C MachXO2 Embedded Configuration Access Demo\r\n");
	printf(    "               Version 1.1.0  Dec 2014\r\n");
	printf(    "===================================================\r\n");
	

	
	while (!done)
	{
		// Pause for a second to show menu/actions of selection
		if (display)
			MicoSleepMilliSecs(200);

		// Show menu (maybe) and get next choice
		selection = menu(display);
		
		display = 1;   // assume user enter valid selection, so redisplay menu after doing this selection
		

		switch (selection)
		{
			case 'T':   // Pico board I2C Temperature sensor access test
				temp_sensor_i2c_test(i2c_ctx, 15);
				break;

			case 'L':  // ECP3 Versa LED Test
				LEDtest(32);
				break;
				
#ifdef ENABLE_ALL_MODES
			case 'M':  // Programming Mode
				if (ProgramMode == XO2ECA_PROGRAM_TRANSPARENT)
					ProgramMode = XO2ECA_PROGRAM_OFFLINE;
				else
					ProgramMode = XO2ECA_PROGRAM_TRANSPARENT;
				break;
				
			case 'K':  // Kick the Config engine into Refresh/User Mode
				// Bypass clears out any stray data and gets SPI/I2C ready for user mode
				XO2ECAcmd_Bypass(&xo2_dev); 
				// Boot design to user mode (i.e. like hitting PROGRAM pin)	
				XO2ECAcmd_Refresh(&xo2_dev);
				break;
				
			case 'C':  // Clear all flash
				// Bypass clears out any stray data and gets SPI/I2C ready for user mode
				XO2ECAcmd_openCfgIF(&xo2_dev, TRANSPARENT_MODE); 
				status = XO2ECAcmd_EraseFlash(&xo2_dev, 0x0f);
				XO2ECAcmd_closeCfgIF(&xo2_dev); 
				XO2ECAcmd_Bypass(&xo2_dev);
				printf("XO2ECAcmd_EraseFlash() = %d\r\n", status); 
				break;

#endif
			case 'V':  // Verify Mode
				if (VerifyMode)
					VerifyMode = 0;
				else
					VerifyMode = XO2ECA_PROGRAM_VERIFY;
				break;


			case 'J':
				printf("\r\n-------------------------------------\r\n");
				printf(    "JEDEC Data Structure Info\r\n");
				printf(    "-------------------------------------\r\n");
				printf("=> XO2 Design: %s\r\n", JEDECDesignList[designNum]);
				XO2ECA_apiJEDECinfo(&xo2_dev, pCfgBits);
				break;


			case '1':   // Use XO2 JEDEC design #1 (LCD number pattern)
				printf("\r\n----------------------------------------\r\n");
				printf(    "Using XO2 design #1 (LCD number pattern)\r\n");
				printf(    "----------------------------------------\r\n");
				pCfgBits  = &XO2_JEDEC_ufm_0123;
				designNum = 1;
				break;

			case '2':   // Use XO2 JEDEC design #2 (LCD word pattern)
				printf("\r\n----------------------------------------\r\n");
				printf(    "Using XO2 design #2 (LCD word pattern)\r\n");
				printf(    "----------------------------------------\r\n");
				pCfgBits  = &XO2_JEDEC_ufm_DEAD;
				designNum = 2;
				break;


            //----------------------------------------------------------------
			//            XO2 I2C Use Case Examples
            //----------------------------------------------------------------
            
			case 'P':   // Program  XO2 Flash areas with JEDEC data using chosen mode
				printf("\r\n--------------------------------------------------------\r\n");
				printf(    "Program XO2 with JEDEC Data Structure\r\n");
				printf("XO2 Design: %s", JEDECDesignList[designNum]);
				if (ProgramMode == XO2ECA_PROGRAM_TRANSPARENT)
					printf("  TRANSPARENT");
				else
					printf("  OFFLINE");
					
				if (VerifyMode)
					printf(" + Verify\r\n");
				else
					printf("\r\n");
				printf(    "--------------------------------------------------------\r\n");
					
				status = XO2ECA_apiProgram(&xo2_dev, 
										   pCfgBits, 
										   VerifyMode | ProgramMode | 
										   XO2ECA_ERASE_PROG_CFG | XO2ECA_ERASE_PROG_UFM | XO2ECA_ERASE_PROG_FEATROW);

						
				if (status != OK)
				{
					printf("ERROR! Failed to Program! err=%d\r\n", status);
					XO2ECA_apiClearXO2(&xo2_dev);   // failed, so reset part to blank state
				}	
				else	
				{		   
					printf("=> Programming Successful.\r\n");
				}	
				UFMPagePattern = 0;
				UFMNextPage = UFM_START_PAGE;   // Start here for user data (beginning is EBR init vals)
				break;

			case 'R':   // Read and display Device Hdw info (USERCODE, TraceID, DevID, status, etc.)
				printf("\r\n------------------------------------------------------------\r\n");
				printf(    "XO2 Device Hdw info (USERCODE, TraceID, DevID, status, etc.)\r\n");
				printf(    "------------------------------------------------------------\r\n");
				status = XO2ECA_apiGetHdwInfo(&xo2_dev, &regInfo);
				printf("DeviceID: 0x%08x\r\n", regInfo.devID);
				printf("USERCODE: 0x%08x\r\n", regInfo.UserCode);
				printf(" TraceID: %x %x %x %x %x %x %x %x\r\n", 
					   regInfo.TraceID[0],
					   regInfo.TraceID[1],
					   regInfo.TraceID[2],
					   regInfo.TraceID[3],
					   regInfo.TraceID[4],
					   regInfo.TraceID[5],
					   regInfo.TraceID[6],
					   regInfo.TraceID[7]
					   );
				status = XO2ECA_apiGetHdwStatus(&xo2_dev, &statusReg);
				printf("Status: 0x%x\r\n", statusReg);
				if (statusReg & 0x01)
					printf("\tDONE\r\n");
			    if (statusReg & 0x02)
				    printf("\tBUSY\r\n");
				if (statusReg & 0x04)
					printf("\tFAIL\r\n");
				printf("\tFlash Check: ");
				switch (statusReg>>4)
				{
					case 0:
						printf("No Err\r\n");
						break;
					case 1:
						printf("ID Err\r\n");
						break;
				    case 2:
						printf("CMD Err\r\n");
						break;
					case 3:
						printf("CRC Err\r\n");
						break;
					case 4:
						printf("Preamble Err\r\n");
						break;
					case 5:
						printf("Abort Err\r\n");
						break;
					case 6:
						printf("Overflow Err\r\n");
						break;
					case 7:
						printf("SDM EOF\r\n");
						break;
				}

				break;

			case 'U':   // Update EBR values from JEDEC (changes design behavior)
				printf("\r\n---------------------------------------------------\r\n");
				printf(    "Update just EBR values in UFM\r\n");
				printf(    "From XO2 Design: %s", JEDECDesignList[designNum]);
				if (ProgramMode == XO2ECA_PROGRAM_TRANSPARENT)
					printf("   TRANSPARENT");
				else
					printf("   OFFLINE");
					
				if (VerifyMode)
					printf(" + Verify\r\n");
				else
					printf("\r\n");
				printf(    "---------------------------------------------------\r\n");
				status = XO2ECA_apiProgram(&xo2_dev, 
										   pCfgBits,   // Use UFM data from this design
										   VerifyMode | ProgramMode | XO2ECA_ERASE_PROG_UFM);
						
				if (status != OK)
					printf("ERROR! Failed to Program! err=%d\r\n", status);	
				else			   
					printf("=> Updating Successful.\r\n");	
								   
				UFMPagePattern = 0;
				UFMNextPage = UFM_START_PAGE;   // Start here for user data (beginning is EBR init vals)
				break;

			case 'E':   // Exchange data in upper pages of UFM sector
				UFMPageDataExchange(&xo2_dev);
				break;

			case 'X':   // Exit code
				// done = 1; // for debugging
				break;
	
			default:
				display = 0;  // user entered invalid selection, so just go get another key stroke
				MicoUart_putC(UARTctx, '\007');	 // ring bell

		}


	}
	
	printf("\r\nExiting.\r\n");
		 
	// This function disables the OpenCores I2C master component instance
	// specified by the device context parameter (ctx). Subsequent OpenCores I2C
	// master operations will fail unless the core is explicitly enabled using the
	// OpencoresI2CMasterEnable function.
	OpenCoresI2CMasterDisable(i2c_ctx );
	
	MicoSleepMilliSecs(500);
	
	return 0;

}


/**
 * Display the user interactive menu for exercising the various operations that can
 * be performed with this demo.  Some routines are for testing the Versa board and
 * I2C connection, and have nothing to do with the XO2.  These are sanity checks.
 * Other routines are used for reading XO2 registers via I2C, programing flash, etc.
 * All these routines use the XO2_ECA APIs to demonstrate the access.
 */
char menu(int display)
{
	unsigned char c;
	int status;

	if (display)
	{
	printf("-------------------------------------------------------------\r\n");
	printf("\r\n\tXO2 ECA Demo Menu\r\n");
	printf(    "\t-----------------\r\n");
	printf("\t[P]rogram with JEDEC\r\n");
	
#ifdef ENABLE_ALL_MODES	
	if (ProgramMode == XO2ECA_PROGRAM_TRANSPARENT)
		printf("\t[M]ode of Program = Transparent\r\n");
	else
		printf("\t[M]ode of Program = Offline\r\n");
#endif			
	if (VerifyMode)
		printf("\t[V]erify Programming = ON\r\n");
	else
		printf("\t[V]erify Programming = OFF\r\n");
			
	printf("\t[U]pdate EBR values\r\n");
	printf("\t[R]ead XO2 registers\r\n");
	printf("\t[E]xchange data in UFM\r\n");
	printf("\t[1] select Number design\r\n");
	printf("\t[2] select Word design\r\n");


	printf("\t[T]emperature Test\r\n");
	printf("\t[L]ED Test\r\n");
	
	MicoUart_putC(UARTctx, '>');
	}

	while(1)
	{
		// Use a raw routine, cause gets(), scanf(), etc. are too expensive
		status = MicoUart_getC(UARTctx, &c);
		if (status == 0)
		{
			if ((c >= 'a') && (c <= 'z'))
				c = (c - 'a') + 'A';   // convert to uppercase (avoid expensive toupper() )
			return(c);
		}
	}
}


/**
 * Write the next available page in the UFM user area (do not over write EBR init val area).
 * with the next set of the incrementing pattern.  Then display the pages to see that it
 * was written correctly, and that previous data is preserved and untouched pages are still
 * all 0's.
 * Can exit out of display mode with 'X'
 * UFM_START_PAGE is where in UFM memory this testing begins.  The lower pages hold the
 * the EBR init values that control what is displayed on the LCD screen: Hex number digits,
 * or words spelled with Hex letter digits.  Starting at the beginning of the UFM would 
 * corrupt the LCD display values, so we start up higher in UFM area.
 * 
 * 
 */
void UFMPageDataExchange(XO2Handle_t *pXO2dev)
{
	int devIndex;
	unsigned char buf[16];
	int i;
	unsigned int pg;
	unsigned char c;
	int status;

	devIndex = pXO2dev->devType;
	// first check if NextUFMPage is still in range
	if (UFMNextPage > XO2DevList[devIndex].UFMpages)
	{
		printf("At end of UFM sector.  Erase UFM with P or U\r\n");
		return;
	}

	// Load the pattern
	for (i = 0; i < 16; i++)
		buf[i] = (unsigned char)UFMPagePattern++;

	status = XO2ECA_apiWriteUFM(pXO2dev,
								UFMNextPage,  // startPg to start writing at
								 1,           // number opf pages to write = 1
								 buf,         // pointer to data to write
								 0);          // erase = FALSE, don't erase UFM before writing	
								 
	if (status != OK)
	{
		printf("XO2ECA_apiWriteUFM() ERR = %d!\r\n", status);
		return;	
	}
	
	printf("Successfully Wrote:\r\n");
		
	readAndDisplayPage(pXO2dev, UFMNextPage);
	
	printf("Press a key to review UFM pages; Q to exit:\r\n");
	
	pg = UFM_START_PAGE;
	do 
	{
		do  // wait for key press
		{
			status = MicoUart_getC(UARTctx, &c);
		}
		while (status != 0);
		
		// display next UFM page
		readAndDisplayPage(pXO2dev, pg);
		++pg;
	} while ((c != 'q') && (c != 'Q') && (pg < XO2DevList[devIndex].UFMpages));
	
	++UFMNextPage;   // NextPage for next time. 
		

}

/**
 * Read the I2C temperature sensor chip on the Pico board as a way to 
 * validate I2C bus operations with a 3rd party device.  If the XO2 is
 * not responding to I2C access then this is a way to check the bus is
 * working and the XO2 has a problem.  
 * Prints the 12 bit register value from reading the temperature.
 * Not converted to Farenhite or Celcius, just a raw value.
 * Placing finger on chip will dynamically change reading.
 */
void temp_sensor_i2c_test(OpenCoresI2CMasterCtx_t *i2c_ctx, int loop)
{
	int status;
	unsigned int address = 0x4A;   // 7 bit address of T101 temp sensor on XO2 Pico Board
	unsigned char data[4];
	int temp;
	
	
	printf("\r\nTest: Pico Board I2C Temp. Sensor Access\r\n");
	
	printf("MicoGetDevice(I2CM_OC_NAME) = %x\r\n", (unsigned int)i2c_ctx);
	
	while (loop)
	{
		// This function issues an I2C start on the I2C bus
		// This function allows you to arbitrate for the bus independently of the read/write.
		status = OpenCoresI2CMasterStart( i2c_ctx );
		if (status != OK)
		{
			printf("ERROR! OpenCoresI2CMasterStart() = %d\n", status);
			return;
		}
		
		
		// This function performs block read. It is contingent on a successful start, that
		// is, ownership of the bus. This function issues a start before performing
		// transactions but does not issue a stop when done. The user application must
		// explicitly issue a stop (OpenCoresI2CMasterStop) when it is ready to
		// relinquish the bus.
		data[0] = 0x00;  // Read Register 0
	    // Send Read the ID command    
		status = OpenCoresI2CMasterWrite(i2c_ctx, address, 1, data);
		if (status != OK)
		{
			printf("ERROR! OpenCoresI2CMasterWrite() = %d\n", status);
			return;
		}
		
		
		data[0] = 0;	
		data[1] = 0;	
		status = OpenCoresI2CMasterRead(i2c_ctx, address, 2, data);
		OpenCoresI2CMasterStop(i2c_ctx );
		if (status != 2)  // asked for 2 bytes
		{
			if (status < 0) 
				printf("ERROR! OpenCoresI2CMasterRead(): Err = %d\n", status);
			else
				printf("ERROR! OpenCoresI2CMasterRead(): Only read %d bytes.\n", status);
		}
		else
		{
			// Display raw 12 bit temperature reading value.
			// Could convert to degrees if get the formula.
			temp = (data[0] << 8) | data[1];
			printf("Temp = %x\r\n", temp);
			
		}
	
		MicoSleepMilliSecs(200);
		--loop;	
	}

}


/**
 * ECP3 LED test.
 * Visual indicator on the ECP3 Versa board that software is working,
 * even if I2C portions are having trouble.
 * The 8 discrete LEDs on the left side light (move) up and down
 * and the segments on the 14 Segment display light, indicating proper
 * functioning ECP3 software/hardware.
 * @param loop number of times to blink LEDs.
 */
void LEDtest(int loop)
{
    unsigned char iValue = 0x1;
    unsigned char iShiftLeft = 1;

	printf("\r\n-------------------------------------\r\n");
    printf("Running ECP3 Versa Board LED Test...\r\n");
	printf("-------------------------------------\r\n");

	/* scroll the LEDs */
	while (loop)
	{
		--loop;
		
		// LEDs on side of board light/move up and down.
		*pLEDs = iValue;
		
		// Segments on inside and out go round and round
		*pSEGs = iValue;


		if (iShiftLeft == 1)
		{
			iValue = iValue << 1;
			if (iValue == 0x80)
			{
				iShiftLeft = 0;
			}
		}
		else
		{
			iValue = iValue >> 1;
			if (iValue == 0)
			{
				iValue = 0x01;
				iShiftLeft = 1;
			}
		}
		

		MicoSleepMilliSecs(100);
		
	}
}



void	readAndDisplayPage(XO2Handle_t *pXO2dev, unsigned int ufmPg)
{
	unsigned char buf[16];
	int i;
	int status;

	status = XO2ECA_apiReadBackUFM(pXO2dev,
								    ufmPg,        // starting page to read from
								     1,           // number opf pages to read = 1
								     buf);        // pointer to data storage area
	if (status != OK)
	{
		printf("XO2ECA_apiReadUFM() ERR!\r\n");
		return;	
	}


	printf("PG[%d]:", ufmPg);
	for (i = 0; i < 16; i++)
		printf("  %x", buf[i]);	
	printf("\r\n");
	
}
