/*
 *  COPYRIGHT (c) 2012 by Lattice Semiconductor Corporation
 *
 * All rights reserved. All use of this software and documentation is
 * subject to the License Agreement located in the file LICENSE.
 */

/** @file oc_i2c_port.c 
 * OpenCores I2C Master Controller Access Routines.
 * This implements the Access Layer to the OpenCores I2C controller that is used by
 * the XO2 ECA commands to read/write bytes across the I2C link to the XO2.
 * The specific read/write routines are wrappers around the MSB OpenCores
 * driver functions to map the generic call from a ECA command to the
 * specific parameters needed by the OpenCores I2C routines, such as
 * device context and slave address.
 * <p>
 * All routines will print run-time diagnostic information if compiled with the
 * define DEBUG_I2C.
 * <P>
 * A note on I2C addressing.  The XO2 configuration logic can be accessed in user mode by setting
 * the lower 2 bits of the address to 2'b00.  The Primary I2C Port needs to be enabled in the EFB and 
 * the EFB needs to be instantiated in the design.  Using the Config address (7'bxxxxx00) will direct
 * the I2C transfers to the configuration mode and not the Wishbone interface.
 * 
 */
 
#include <stdio.h>

#include "oc_i2c_port.h"
#include "MicoUtils.h"
#include "systypes.h"

/**
 * Call OpenCores routine to issue an I2C Start bus action.
 * @param pDrvrParams pointer that holds specific information for the I2C driver routine, previously stored in XO2Handle_t.
 * @return OK if arbitrated and got the bus, ERROR if unsuccessful
 */
int oc_i2c_start(void *pDrvrParams)
{
	int status;
	OC_I2C_t *pI2C;

	pI2C = (OC_I2C_t  *)pDrvrParams;

	status = OpenCoresI2CMasterStart( pI2C->i2c_ctx );
#ifdef DEBUG_I2C
	printf("OpenCoresI2CMasterStart() = %d\r\n", status);
#endif
	if (status != OK)
		return(ERROR);
	else
		return(OK);

}




/**
 * Call OpenCores routine to issue an I2C Stop bus action.
 * @param pDrvrParams pointer that holds specific information for the I2C driver routine, previously stored in XO2Handle_t.
 * @return always returns OK
 */
int oc_i2c_stop(void *pDrvrParams)
{
	OC_I2C_t *pI2C;

	pI2C = (OC_I2C_t  *)pDrvrParams;

	OpenCoresI2CMasterStop( pI2C->i2c_ctx );
#ifdef DEBUG_I2C
	printf("OpenCoresI2CMasterStop()\r\n");
#endif
	return(OK);  // no return code from this stop()

}


/**
 * Call OpenCores routine to issue an I2C Read bus action.
 * @param pDrvrParams pointer that holds specific information for the I2C driver routine, previously stored in XO2Handle_t.
 * @param len nubmer of bytes to read
 * @param pBuf storage for bytes read
 * @return OK if arbitrated and got the bus, ERROR if command failed or did not read all requested bytes
 */
int oc_i2c_read(void *pDrvrParams, unsigned int len, unsigned char *pBuf)
{
	int status;
	OC_I2C_t *pI2C;

	pI2C = (OC_I2C_t  *)pDrvrParams;

	status = OpenCoresI2CMasterRead(pI2C->i2c_ctx, 
					pI2C->slaveAddr, 
					len, 
					pBuf);
#ifdef DEBUG_I2C
	printf("OpenCoresI2CMasterRead() = %d\r\n", status);
#endif
	if (status < OK)
		return(ERROR);
	else if (status != len)   // make sure read all we asked for
		return(ERROR);
	else
		return(OK);

}


/**
 * Call OpenCores routine to issue an I2C Write bus action.
 * @param pDrvrParams pointer that holds specific information for the I2C driver routine, previously stored in XO2Handle_t.
 * @param len number of bytes to write, stored in pBuf 
 * @param pBuf storage for bytes to write
 * @return OK if arbitrated and got the bus, ERROR if command failed (did not get ACK)
 */
int oc_i2c_write(void *pDrvrParams, unsigned int len, unsigned char *pBuf)
{
	int status;
	OC_I2C_t *pI2C;

	pI2C = (OC_I2C_t  *)pDrvrParams;

	status = OpenCoresI2CMasterWrite(pI2C->i2c_ctx, 
					pI2C->slaveAddr, 
					len, 
					pBuf);
#ifdef DEBUG_I2C
	printf("OpenCoresI2CMasterWrite() = %d\r\n", status);
#endif
	if (status != OK)
		return(ERROR);
	else
		return(OK);

}
